<?php
/*
| ----------------------------------------------------
| File        : class-main.php
| Project     : Special Recent Posts plugin for Wordpress
| Version     : 1.6.4
| Description : This file contains the default global
|               enviroment values
| Author      : Luca Grandicelli
| Author URL  : http://www.lucagrandicelli.com
| Plugin URL  : http://www.lucagrandicelli.com/special-recent-posts-plugin-for-wordpress/
| ----------------------------------------------------
*/

class SpecialRecentPosts {

/*
| ---------------------------------------------
| CLASS PROPERTIES
| ---------------------------------------------
*/

	// Declaring default plugin options array.
	private $plugin_args;
	
	// Declaring widget instance options array.
	private $widget_args;
	
	// Declaring Single Post ID (the current one displayed when in single post view).
	private $singleID;

/*
| ---------------------------------------------
| CLASS CONSTRUCTOR & DECONSTRUCTOR
| ---------------------------------------------
*/
	// Class Constructor.
	// In this section we define the plugin global admin values and assign the selected widget values.
	public function __construct($args = array()) {

		// Setting up plugin options to be available throughout the plugin.
		$this->plugin_args  = get_option('srp_plugin_options');
		
		// Including external widget values.
		global $srp_default_widget_values;
		
		// Setting up widget options to be available throughout the plugin.
		$this->widget_args = array_merge($srp_default_widget_values, $args);
		
		// Updating thumbnails sizes if these are not set on custom PHP call. Assign default value.
		if (!isset($this->widget_args["srp_thumbnail_wdg_width"])) {
			$this->widget_args["srp_thumbnail_wdg_width"] = $this->plugin_args["srp_thumbnail_width"];
		}
		
		if (!isset($this->widget_args["srp_thumbnail_wdg_height"])) {
			$this->widget_args["srp_thumbnail_wdg_height"] = $this->plugin_args["srp_thumbnail_height"];
		}
		
		if (is_single()) {
			global $post;
			$this->singleID = $post->ID;
		}
	}
	
	// Class Deconstructor.
	public function __deconstruct() {}

/*
| ---------------------------------------------
| STATIC METHODS
| ---------------------------------------------
*/

	// This method handles all the actions for the plugin to be initialized.
	static function install_plugin() {
		
		// Loading text domain for translations.
		load_plugin_textdomain(SRP_TRANSLATION_ID, false, dirname( plugin_basename(__FILE__)) . SRP_LANG_FOLDER);
		
		// Importing global default options array.
		global $srp_default_plugin_values;
		
		// Creating WP Option with default values.
		add_option('srp_plugin_options', $srp_default_plugin_values, '', 'no');
	}
	
	// This method handles all the actions for the plugin to be uninstalled.
	static function uninstall_plugin() {
		
		// Deleting main WP Option.
		delete_option('srp_plugin_options');
	}

/*
| ---------------------------------------------
| CLASS MAIN METHODS
| ---------------------------------------------
*/
	
	// Method to retrieve first post image url.
	private function getFirstImageUrl($thumb_width, $thumb_height) {
	
		// Including global WP Enviroment.
		global $post, $posts;
		
		//Getting images attached to the post.
		$args = array(
			'post_type'      => 'attachment',
			'post_mime_type' => 'image',
			'numberposts'    => -1,
			'order'          => 'ASC',
			'post_status'    => null,
			'post_parent'    => $post->ID
		);
		
		$attachments = get_posts($args);
		
		// Check for attachments.
		if ($attachments) {
			
			// Cycling through attachments.
			foreach ($attachments as $attachment) {
			
				// Retrieving image url.
				$imgabs = base64_encode(get_attached_file($attachment->ID));
				break;
			}
			
			//Return image tag using adaptive resize with PHPThumb.
			return "<img class=\"srp-widget-thmb\" src=\"" . SRP_PLUGIN_URL . SRP_THUMBPHP_HANDLER . "?width=" . $thumb_width . "&height=" . $thumb_height . "&display_mode=adaptive&file=" . $imgabs . "\" />";
			
		} else {
			// there are no attachment for the current post. Switch to default thumbnail image.
			return $this->displayDefaultThumb($this->widget_args["srp_thumbnail_wdg_width"], $this->widget_args["srp_thumbnail_wdg_height"]);
		}
	}
	
	// Method to display default thumbnail.
	private function displayDefaultThumb($thumb_width, $thumb_height) {
	
		// Check if a custom thumbnail url has been provided.
		if ($this->plugin_args['srp_thumbnail_url'] != '') {
			
			// Returning custom default thumbnail image.
			return "<img class=\"srp-widget-thmb\" src=\"" . $this->plugin_args['srp_thumbnail_url'] . "\" width=\"" . $thumb_width . "\" height=\"" . $thumb_height . "\" />";
			
		} else {
			// Returning default thumbnail image.
			return "<img class=\"srp-widget-thmb\" src=\"" . SRP_PLUGIN_URL . SRP_DEFAULT_THUMB . "\" width=\"" . $thumb_width . "\" height=\"" . $thumb_height . "\" />";
		}
	}
	
	// Main method to extract and elaborate post excerpt based on user choices.
	private function extractExcerpt($post, $excerpt_length, $excerpt_length_mode) {
		
		// Check for "cut mode".
		switch($excerpt_length_mode) {
		
			case 'words':
				
				// Check if excerpt is available.
				if ($post->post_excerpt) {
				
					// Return normal excerpt using 'words cut'.
					return $this->substrWords($this->srp_sanitize($post->post_excerpt), $excerpt_length);
					
				} else {
				
					// Normal excerpt not available. Retrieve text from post content using 'words cut'.
					return $this->substrWords($this->srp_sanitize($post->post_content), $excerpt_length);
				}
				
			break;
			
			case 'chars':
				
				// Check if excerpt is available.
				if ($post->post_excerpt) {
				
					// Return normal excerpt using 'characters cut'.
					return substr($this->srp_sanitize($post->post_excerpt), 0, $excerpt_length);
					
				} else {
				
					// Normal excerpt not available. Retrieve text from post content using 'characters cut'.
					return substr($this->srp_sanitize($post->post_content), 0, $excerpt_length);
				}
				
			break;
			
			case 'fullexcerpt':
				
				// Check if excerpt is available.
				if ($post->post_excerpt) {
				
					// Return normal excerpt using 'characters cut'.
					return $this->srp_sanitize($post->post_excerpt);
					
				} else {
				
					// Normal excerpt not available. Retrieve text from post content using 'characters cut'.
					return substr($this->srp_sanitize($post->post_content), 0, $excerpt_length);
				}
				
			break;
		}
	}
	
	// Main method to extract and elaborate post title based on user choices.
	private function extractTitle($post, $title_length, $title_length_mode) {
		
		// Check for "cut mode".
		switch($title_length_mode) {
		
			case 'words':
			
				// Return normal title using 'words cut'.
				return $this->substrWords($this->srp_sanitize($post->post_title), $title_length);
			break;
			
			case 'chars':
			
				// Return normal title using 'characters cut'.
				return substr($this->srp_sanitize($post->post_title), 0, $title_length);
			break;
			
			case 'fulltitle':
			
				// Return normal title using 'characters cut'.
				return $this->srp_sanitize($post->post_title);
			break;
		}
	}

	// Main method to retrieve posts.
	private function getPosts() {
		
		// Defining args array.
		$args = array (
			'post_type'   => $this->widget_args["srp_post_type"],
			'numberposts' => $this->widget_args["srp_number_post_option"],
			'order'       => $this->widget_args["srp_order_post_option"]
		);
		
		// Check if category filter is applied.
		if ($this->widget_args["srp_filter_cat_option"] != '-1') {
			
			// Filter result posts by category ID.
			$args["category"] = $this->widget_args["srp_filter_cat_option"];
		}
		
		// Check if post offset option is enabled.
		if ($this->plugin_args["srp_post_offset"] == 'yes') {
		
			// Filter result posts by category ID.
			$args["exclude"] = $this->singleID;
		}
		
		// Check if exclude posts option is applied.
		if (!empty($this->widget_args["srp_exclude_option"])) {
			
			// Filter result posts by category ID.
			$args["exclude"] = $this->widget_args["srp_exclude_option"];
		}
		
		// Check if include posts option is applied.
		if (!empty($this->widget_args["srp_include_option"])) {
			
			// Filter result posts by category ID.
			$args["include"] = $this->widget_args["srp_include_option"];
		}

		// Calling built-in Wordpress 'get_posts' function.
		$result_posts = get_posts($args);
		
		// Checking if result posts array is empty.
		if (empty($result_posts)) {
		
			// No recent posts available. Return empty array.
			return $result_posts;
		}
		
		// Check if random posts option is checked.
		if ($this->widget_args["srp_orderby_post_option"] != "") {
			
			// Let's shuffle the result array.
			shuffle($result_posts);
		}

		// Return result array.
		return $result_posts;
	}
	
	// Main method to display posts.
	public function displayPosts($widget_call = NULL) {
	
		// Declaring global $post variable.
		global $post;

		// Open Widget Container.
		echo "<div class=\"srp-widget-container\">";
		
		// Check if this method has been called from a widget or from a direct PHP call.
		if (!$widget_call) {
		
			// Check if SRP is displaying a category filter result and if it should use the linked category title.
			if ( ($this->widget_args["srp_filter_cat_option"] != '-1') && ($this->plugin_args["srp_category_title"] == "yes") ) {
				
				// Get Category Link.
				$srp_category_link = get_category_link($this->widget_args["srp_filter_cat_option"]);
				
				// Display Category Title.
				echo "<h3 class=\"widget-title\"><a rel=\"nofollow\" href=\"" . $srp_category_link . "\">" . get_cat_name($this->widget_args["srp_filter_cat_option"]) . "</a></h3>";
				
			} else {
			
				// Display Widget Title.
				echo "<h3 class=\"widget-title\">" . $this->srp_sanitize($this->widget_args["srp_widget_title"]) . "</h3>";
			}
		}
		
		// Fetching recent posts.
		$recent_posts = $this->getPosts();
		
		// Checking if posts are available.
		if (empty($recent_posts)) {
		
			// No posts available. Display message.
			echo "<p>" . $this->srp_sanitize($this->plugin_args['srp_noposts_message']) . "</p>";
			
		} else {
			
			// Recent posts are available. Cycle through result posts.
			foreach($this->getPosts() as $post) {
			
				// Prepare access to all post data.
				setup_postdata($post);
				
				// Single Post Container.
				echo "<div class=\"srp-widget-singlepost\">";
				
				// Check if thumbnail option has been set to true.
				if ($this->widget_args["srp_thumbnail_option"] == 'yes') {
					
					// Thumbnail box.
					echo "<div class=\"srp-thumbnail-box\">";
				
					// Check if 'Thumbnail Mode' is supported by the current theme and post has a related thumbnail.
					if (function_exists('has_post_thumbnail') && has_post_thumbnail()) {
					
						// Retrieving thumbnail src attribute.
						$thumbabs = base64_encode(get_attached_file(get_post_thumbnail_id($post->ID)));

						// Check for thumbnail link option.
						if ('yes' == $this->plugin_args['srp_thumbnail_link']) {
						
							// Display thumbnail with link to post.
							echo "<a class=\"srp-widget-thmblink\" href=\"" . get_permalink($post->ID) . "\" title=\"" . the_title_attribute(array('echo' => 0)) . "\" rel=\"nofollow\">";
							echo "<img class=\"srp-widget-thmb\" src=\"" . SRP_PLUGIN_URL . SRP_THUMBPHP_HANDLER . "?width=" . $this->widget_args["srp_thumbnail_wdg_width"] . "&height=" . $this->widget_args["srp_thumbnail_wdg_height"] . "&display_mode=adaptive&file=" . $thumbabs . "\" />";
							echo "</a>";
						
						} else {
						
							// Display post thumbnail without link.
							echo "<img class=\"srp-widget-thmb\" src=\"" . SRP_PLUGIN_URL . SRP_THUMBPHP_HANDLER . "?width=" . $this->widget_args["srp_thumbnail_wdg_width"] . "&height=" . $this->widget_args["srp_thumbnail_wdg_height"] . "&display_mode=adaptive&file=" . $thumbabs . "\" />";
						}
						
					} else {
					
						// Current theme doesn't support post thumbnail or current post has no thumbnail associated. Let's switch the entire process on using extracted first post images.
						if ('yes' == $this->plugin_args['srp_thumbnail_link']) {
						
							// Display thumbnail with link to post.
							echo "<a class=\"srp-widget-thmblink\" href=\"" . get_permalink($post->ID) . "\" title=\"" . the_title_attribute(array('echo' => 0)) . "\" rel=\"nofollow\">";
							echo $this->getFirstImageUrl($this->widget_args["srp_thumbnail_wdg_width"], $this->widget_args["srp_thumbnail_wdg_height"]);
							echo "</a>";
							
						} else {
						
							// Display post thumbnail without link.
							echo $this->getFirstImageUrl($this->widget_args["srp_thumbnail_wdg_width"], $this->widget_args["srp_thumbnail_wdg_height"]);
						}
					}
					
					// EOF Thumbnail box.
					echo "</div>";
				}
				
				// BOF Content Box.
				echo "<div class=\"srp-content-box\">";
				
				// Print Post Title.
				echo "<h4 class=\"srp-widget-title\">";
				echo "<a class=\"srp-widget-title-link\" rel=\"nofollow\" href=\"" . get_permalink($post->ID) . "\" title=\"" . the_title_attribute(array('echo' => 0)) . "\">" . $this->extractTitle($post, $this->plugin_args['srp_title_length'], $this->plugin_args['srp_title_length_mode']) . "</a>";
				echo "</h4>";
				
				// Check for post date option.
				if ('yes' == $this->widget_args["srp_post_date_option"]) {
				
					// Output post date with selected arguments.
					echo "<span class=\"srp-widget-date\">" . get_the_time($this->plugin_args['srp_date_content']) . "</span>";
				}
				
				// Check for Post Content Option.
				if ('titleexcerpt' == $this->widget_args["srp_content_post_option"]) {
					
					// Print Excerpt.
					echo "<p class=\"srp-widget-excerpt\">" . $this->extractExcerpt($post, $this->plugin_args['srp_excerpt_length'], $this->plugin_args['srp_excerpt_length_mode']);
					
					// Check if there is an image stringbreak
					if ($this->plugin_args['srp_image_string_break'] != "") {
						
						// Display image button.
						echo "<a class=\"srp-widget-stringbreak-link-image\" href=\"" . get_permalink($post->ID) . "\" title=\"" . the_title_attribute(array('echo' => 0)) . "\" rel=\"nofollow\"><img class=\"srp-widget-stringbreak-image\" src=\"" . $this->srp_sanitize($this->plugin_args['srp_image_string_break']) . "\" /></a></p>";
					
					} elseif ($this->plugin_args['srp_string_break'] != "") {
					
						// Use a text stringbreak. Check if string break should be linked to post.
						if ('yes' == $this->plugin_args['srp_string_break_link']) {
						
							// Print string break with link to post.
							echo "<a class=\"srp-widget-stringbreak-link\" href=\"" . get_permalink($post->ID) . "\" title=\"" . the_title_attribute(array('echo' => 0)) . "\" rel=\"nofollow\">" . $this->srp_sanitize($this->plugin_args['srp_string_break']) . "</a></p>";
						} else {
							
							// Print string break without link to post.
							echo "<span class=\"srp-widget-stringbreak\">" . $this->srp_sanitize($this->plugin_args['srp_string_break']) . "</span></p>";
						}
					}
				}
				
				// EOF Content Box.
				echo "</div>";
				
				// Closing Single Post Container.
				echo "</div>";
				
			} // EOF foreach cycle.
			
			// Reset $post data array.
			wp_reset_postdata();
			
		} // EOF Empty posts check.
		
		echo "</div>"; // Closing Widget Container.
	}
	
/*
| ---------------------------------------------
| UTILITY METHODS
| In this section we collect several utility
| general methods.
| ---------------------------------------------
*/
	// This method sanitize strings output.
	private function srp_sanitize($string) {
		
		// We need to remove all the exceeding stuff. Remove shortcodes and slashes.
		$temp_output = trim(stripslashes(strip_shortcodes($string)));
		
		// Applying qTranslate Filter if this exists.
		if (function_exists('qtrans_useCurrentLanguageIfNotFoundShowAvailable')) {
			$temp_output = qtrans_useCurrentLanguageIfNotFoundShowAvailable($temp_output);
		}
		
		// If there is a tag filter, keep them separated from strip_tags.
		if (!empty($this->plugin_args["srp_excerpt_allowed_tags"])) {
		
			// Strip tags except the ones specified.
			return strip_tags($temp_output, htmlspecialchars_decode($this->plugin_args["srp_excerpt_allowed_tags"]));
			
		} else {
		
			// Otherwise completely strip tags from text.
			return strip_tags($temp_output);
		}
	}
	
	// This method uses the same logic of PHP function 'substr', but works with words instead of characters.
	private function substrWords($str, $n) {
		
		// Check if max length is equal to original string length. In that case, return the string without making any 'cut'.
		if (str_word_count($str, 0) > $n) {

			// Uses PHP 'str_word_count' function to extract total words and put them into an array.
			$w = explode(" ", $str);
			
			// Lets' cut the array using our max length variable ($n).
			array_splice($w, $n);
			
			// Re-convert array to string and return.
			return implode(" ", $w);
			
		} else {
			
			// Return string as it is, without making any 'cut'.
			return $str;
		}
	}
} // EOF Class.